;; Generated package description from denote-silo.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "denote-silo" "0.2.0" "Convenience functions for using Denote in multiple silos" '((emacs "28.1") (denote "4.0.0")) :commit "5710b3f15e477c1b2f71ddb9c8cf719f9589f1cb" :authors '(("Protesilaos Stavrou" . "info@protesilaos.com")) :maintainer '("Protesilaos Stavrou" . "info@protesilaos.com") :url "https://github.com/protesilaos/denote-silo")
